==> smasher-1.0.5-source.tar.gz: smasher-1.0.5/src/wav16smash.cpp <==
#include <algorithm>
#include <iostream>
#include <sndfile.hh>
#include "wav16smash.h"
#include "keymapper.h"
#include "beatfinder.h"
#include "samplemath.h"
#include <stdexcept>

Wav16Smash::Wav16Smash ( bool *abort, std::string *message, size_t crossFadeRangeParameter )
{
    smashMapper = NULL;
    abortPtr = abort;
    messagePtr = message;

    crossFadeRange = crossFadeRangeParameter;
    setState ( WAV16_INIT );
}

Wav16Smash::~Wav16Smash()
{
    delete smashMapper;
}

bool Wav16Smash::checkAbort()
{
    if ( *abortPtr )
        return true;

    return false;
}

void Wav16Smash::clear()
{
    dataIn.clear();
    dataOut.clear();
    clearSliceInfo();
    clearEffectsInfo();
    setState ( WAV16_INIT );
}

void Wav16Smash::clearSliceInfo()
{
    // todo: delete pointers
    sliceOffsetVector.clear();
    setSmashMapper ( NULL );
}

void Wav16Smash::clearEffectsInfo()
{
    size_t limit;

    limit = sliceVectorTransformerVector.size();
    for ( size_t i = 0; i < limit; i++ )
    {
        delete sliceVectorTransformerVector.at ( i );
    }
    sliceVectorTransformerVector.clear();

    limit = sliceTransformerVector.size();
    for ( size_t i = 0; i < limit; i++ )
    {
        delete sliceTransformerVector.at ( i );
    }
    sliceTransformerVector.clear();
}


bool Wav16Smash::getSlice ( size_t sliceNo, std::vector<double>& slice, size_t gateRange )
{
    if ( getState() != WAV16_HAS_DATA_OUT )
        return false;

    if ( ! ( sliceNo < sliceOffsetVector.size() ) )
        return false;

    size_t offset, length, limit;
    offset = sliceOffsetVector.at ( sliceNo ).first;
    length = sliceOffsetVector.at ( sliceNo ).second;
    limit = offset + length;

    size_t zeroRange = 64;

    size_t i = offset;

    while ( i < offset + zeroRange )
    {
        slice.push_back ( 0.0 );
        i++;
    }
    while ( i < offset + zeroRange + gateRange )
    {
        slice.push_back ( lConcaveSoftClipper ( i, 0, gateRange ) * dataOut.at(i));
        i++;
    }
    while ( i < limit - gateRange )
    {
        slice.push_back ( dataOut.at(i));
        i++;
    }
    while ( i < length - zeroRange )
    {
        slice.push_back ( rConvexSoftClipper ( i, limit - gateRange + i, limit ) * dataOut.at(i));
        i++;
    }

    while ( i < length )
    {
        slice.push_back ( 0.0 );
        i++;
    }

    return true;
}

bool Wav16Smash::smash ( size_t requestedSliceNo )
{
    if ( checkAbort() )
        return false;

    int state = getState();
    if ( state != WAV16_HAS_DATA_IN && state != WAV16_HAS_DATA_OUT )
    {
        *messagePtr = "can't slice loop";
        return false;
    }

    dataOut.clear();

    if ( !channels )
    {
        setState (WAV16_ERROR);
        *messagePtr = "0 channels";
        return false;
    }

    if ( !requestedSliceNo )
    {
        setState (WAV16_ERROR);
        *messagePtr = "0 slices";
        return false;
    }

    if ( dataIn.size() % channels )
    {
        setState (WAV16_ERROR);
        *messagePtr = "channels not balanced";
        return false;
    }

    else if ( smashMapper == NULL )
    {
        setState (WAV16_ERROR);
        *messagePtr = "can't slice loop";
        return false;
    }

    size_t sliceSize = dataIn.size() / requestedSliceNo;
    if ( crossFadeRange > sliceSize / channels )
    {
        setState (WAV16_HAS_DATA_IN);
        *messagePtr = "can't smash loop into requested number of slices";
        return false;
    }

    if ( checkAbort() )
        return false;

    std::vector<double> dataInL, dataInR;
    std::vector<std::vector<double> > sliceVectorL, sliceVectorR;

    bool retL, retR = false;

    switch ( channels )
    {
    case 1:
        retL = dataToSliceVectorCrossFade ( &dataIn, sliceVectorL, requestedSliceNo );
        retR = true;
        break;
    case 2:
        SampleMath::splitChannels ( &dataIn, &dataInL, &dataInR);
        retL = dataToSliceVectorCrossFade ( &dataInL, sliceVectorL, requestedSliceNo );
        retR = dataToSliceVectorCrossFade ( &dataInR, sliceVectorR, requestedSliceNo );
        break;
    default:
        SampleMath::splitChannels ( &dataIn, &dataInL, &dataInR);
        retL = dataToSliceVectorCrossFade ( &dataInL, sliceVectorL, requestedSliceNo );
        retR = dataToSliceVectorCrossFade ( &dataInR, sliceVectorR, requestedSliceNo );
        break;
    }

    if ( !( retL && retR ) )
        return false;

    if ( checkAbort() )
        return false;

    switch ( channels )
    {
    case 1:
        smashMapper->map ( sliceVectorL );
        break;
    case 2:
        smashMapper->map ( sliceVectorL );
        smashMapper->map ( sliceVectorR );
        break;
    default:
        smashMapper->map ( sliceVectorL );
        smashMapper->map ( sliceVectorR );
        break;
    }

    if ( checkAbort() )
        return false;

    std::vector<SliceVectorTransformer<double> *>::iterator svit;
    for ( svit = sliceVectorTransformerVector.begin(); svit != sliceVectorTransformerVector.end(); svit++)
    {
        (*svit)->vectorTransform ( &sliceVectorL, ( channels == 2 ) ? &sliceVectorR : NULL);

        if ( checkAbort() )
            return false;
    }

    std::vector<double> dataOutL, dataOutR;

    switch ( channels )
    {
    case 1:
        retL = sliceVectorToDataCrossFade( sliceVectorL, &dataOut);
        retR = true;
        break;
    case 2:
        retL = sliceVectorToDataCrossFade( sliceVectorL, &dataOutL);
        retR = sliceVectorToDataCrossFade( sliceVectorR, &dataOutR);
        break;
    default:
        retL = sliceVectorToDataCrossFade( sliceVectorL, &dataOutL);
        retR = sliceVectorToDataCrossFade( sliceVectorR, &dataOutR);
        break;
    }

    if ( checkAbort() )
        return false;

    if ( !( retL && retR ) )
        return false;

    std::vector<SliceTransformer<double> *>::iterator stit;
    for ( stit = sliceTransformerVector.begin(); stit != sliceTransformerVector.end(); stit++ )
    {
        (*stit)->transform ( &dataOutL, ( channels == 2 ) ? &dataOutR : NULL );

        if ( checkAbort() )
            return false;
    }

    if ( channels == 2 )
        SampleMath::joinChannels ( &dataOutL, &dataOutR, &dataOut );

    setState ( WAV16_HAS_DATA_OUT );
    return true;
}

bool Wav16Smash::dataToSliceVectorCrossFade ( std::vector<double> *data, std::vector<std::vector<double> >& sliceVector, size_t requestedSliceNo )
{
    std::vector<double> currentSlice, lastSlice;
    size_t dataSize = data->size();
    size_t sliceSize = dataSize / requestedSliceNo;
    size_t offset, limit;
    offset = limit = 0;
    sliceVector.clear();

    if ( requestedSliceNo > 1 )
    {

        for ( size_t i = 0; i < requestedSliceNo - 1; i ++ )
        {
            limit = offset + sliceSize + crossFadeRange;
            for ( size_t j = offset; j < limit; j++)
                currentSlice.push_back ( data->at ( j ) );

            sliceVector.push_back ( currentSlice );
            currentSlice.clear();
            offset += sliceSize;
        }

    }

    for ( size_t i = offset; i < dataSize; i++)
        lastSlice.push_back ( data->at ( i ) );

    for ( size_t i = 0; i < crossFadeRange; i++)
        lastSlice.push_back ( data->at ( i ) );
    sliceVector.push_back ( lastSlice );

    return true;
}


bool Wav16Smash::sliceVectorToDataCrossFade ( std::vector<std::vector<double> >& sliceVector, std::vector<double> *data )
{
    std::vector<std::vector<double> >::iterator vit;
    size_t dataSize, sliceVectorSize, offset, length, limit, i;
    dataSize = offset = length = limit = 0;
    sliceVectorSize = sliceVector.size();
    sliceOffsetVector.clear();

    for ( vit = sliceVector.begin(); vit != sliceVector.end(); vit++ )
        dataSize += vit->size() - crossFadeRange;

    for ( i = 0; i < dataSize; i++ )
        data->push_back ( 0.0 );

    for ( size_t j = 0; j < sliceVectorSize; j++ )
    {
        length = sliceVector.at ( j ).size();
        limit = length - crossFadeRange;

        i = 0;

        while ( i < crossFadeRange )
        {
            (*data )[ offset + i ] +=
                ( j > 0 ) ?
                lConcaveSoftClipper( i, 0, crossFadeRange ) * sliceVector.at ( j ).at( i ) :
                lConcaveSoftClipper( i, 0, 128 ) * sliceVector.at ( j ).at( i );
            i++;
        }


        while ( ( i >= crossFadeRange ) && ( i < limit  ) )
        {
            (*data )[ offset + i ] += sliceVector.at ( j ).at( i );
            i++;
        }

        while ( i < length )
        {
            (*data )[ offset + i ] +=
                ( j < sliceVectorSize - 1 ) ?
                rConvexSoftClipper ( i, limit, length ) * sliceVector.at ( j ).at( i ):
                rConvexSoftClipper ( i, length - 128, length ) * sliceVector.at ( j ).at( i );
            i++;
        }

        sliceOffsetVector.push_back ( std::make_pair ( channels * offset, channels * limit ) );
        offset += limit;
    }
    return true;

}


void Wav16Smash::addSliceVectorTransformer (SliceVectorTransformer<double> *sliceVectorTransformer)
{
    int state = getState();
    if ( state != WAV16_HAS_DATA_IN && state != WAV16_HAS_DATA_OUT )
        return;

    sliceVectorTransformerVector.push_back ( sliceVectorTransformer );
}


void Wav16Smash::addSliceTransformer ( SliceTransformer<double> *sliceTransformer )
{
    int state = getState();
    if ( state != WAV16_HAS_DATA_IN && state != WAV16_HAS_DATA_OUT )
        return;

    sliceTransformerVector.push_back ( sliceTransformer );
}

==> smasher-1.0.8-source.tar.gz: smasher-1.0.8/src/wav16smash.cpp <==
#include <algorithm>
#include <iostream>
#include <sndfile.hh>
#include "wav16smash.h"
#include "keymapper.h"
#include "beatfinder.h"
#include "samplemath.h"
#include <stdexcept>

Wav16Smash::Wav16Smash ( bool *abort, std::string *message, size_t crossFadeRangeParameter )
{
    smashMapper = NULL;
    abortPtr = abort;
    messagePtr = message;

    crossFadeRange = crossFadeRangeParameter;
    setState ( WAV16_INIT );
}

Wav16Smash::~Wav16Smash()
{
    //delete smashMapper; // don't delete: belongs to parent
}

bool Wav16Smash::checkAbort()
{
    if ( *abortPtr )
        return true;

    return false;
}

void Wav16Smash::clear()
{
    dataIn.clear();
    dataOut.clear();
    clearSliceInfo();
    clearEffectsInfo();
    setState ( WAV16_INIT );
}

void Wav16Smash::clearSliceInfo()
{
    sliceOffsetVector.clear();
    setSmashMapper ( NULL );
}

void Wav16Smash::clearEffectsInfo()
{
    size_t limit;

    limit = sliceVectorTransformerVector.size();
    for ( size_t i = 0; i < limit; i++ )
    {
        delete sliceVectorTransformerVector.at ( i );
    }
    sliceVectorTransformerVector.clear();

    limit = sliceTransformerVector.size();
    for ( size_t i = 0; i < limit; i++ )
    {
        delete sliceTransformerVector.at ( i );
    }
    sliceTransformerVector.clear();
}


bool Wav16Smash::getSlice ( size_t sliceNo, std::vector<double>& slice, size_t gateRange )
{
    if ( getState() != WAV16_HAS_DATA_OUT )
        return false;

    if ( ! ( sliceNo < sliceOffsetVector.size() ) )
        return false;

    size_t offset, length, limit, dataSize;
    offset = sliceOffsetVector.at ( sliceNo ).first;
    length = sliceOffsetVector.at ( sliceNo ).second;
    limit = offset + length;
    dataSize = dataOut.size();

    slice.clear();

    for ( size_t i = offset; i < limit; i++ )
        slice.push_back ( dataOut.at ( i ) );

    size_t sliceSize = slice.size();

    if ( sliceSize < gateRange * 3 )
        return false;

    for ( size_t j = 0; j < gateRange; j++ )
        slice[j] = lConcaveSoftClipper ( j, 0, gateRange ) * slice.at ( j );

    for ( size_t k = sliceSize - gateRange; k < sliceSize; k++ )
        slice[k] = rConvexSoftClipper ( k, sliceSize - gateRange, sliceSize ) * slice.at ( k );

    return true;
}

bool Wav16Smash::smash ( size_t requestedSliceNo )
{
    if ( checkAbort() )
        return false;

    int state = getState();
    if ( state != WAV16_HAS_DATA_IN && state != WAV16_HAS_DATA_OUT )
    {
        *messagePtr = "can't slice loop";
        return false;
    }

    dataOut.clear();

    if ( !channels )
    {
        setState (WAV16_ERROR);
        *messagePtr = "0 channels";
        return false;
    }

    if ( !requestedSliceNo )
    {
        setState (WAV16_ERROR);
        *messagePtr = "0 slices";
        return false;
    }

    if ( dataIn.size() % channels )
    {
        setState (WAV16_ERROR);
        *messagePtr = "channels not balanced";
        return false;
    }

    else if ( smashMapper == NULL )
    {
        setState (WAV16_ERROR);
        *messagePtr = "can't slice loop";
        return false;
    }

    size_t sliceSize = dataIn.size() / requestedSliceNo;
    if ( crossFadeRange > sliceSize / channels )
    {
        setState (WAV16_HAS_DATA_IN);
        *messagePtr = "can't smash loop into requested number of slices";
        return false;
    }

    if ( checkAbort() )
        return false;

    std::vector<double> dataInL, dataInR;
    std::vector<std::vector<double> > sliceVectorL, sliceVectorR;

    bool retL, retR = false;

    switch ( channels )
    {
    case 1:
        retL = dataToSliceVectorCrossFade ( &dataIn, sliceVectorL, requestedSliceNo );
        retR = true;
        break;
    case 2:
        SampleMath::splitChannels ( &dataIn, &dataInL, &dataInR);
        retL = dataToSliceVectorCrossFade ( &dataInL, sliceVectorL, requestedSliceNo );
        retR = dataToSliceVectorCrossFade ( &dataInR, sliceVectorR, requestedSliceNo );
        break;
    default:
        SampleMath::splitChannels ( &dataIn, &dataInL, &dataInR);
        retL = dataToSliceVectorCrossFade ( &dataInL, sliceVectorL, requestedSliceNo );
        retR = dataToSliceVectorCrossFade ( &dataInR, sliceVectorR, requestedSliceNo );
        break;
    }

    if ( !( retL && retR ) )
        return false;

    if ( checkAbort() )
        return false;

    switch ( channels )
    {
    case 1:
        smashMapper->map ( sliceVectorL );
        break;
    case 2:
        smashMapper->map ( sliceVectorL );
        smashMapper->map ( sliceVectorR );
        break;
    default:
        smashMapper->map ( sliceVectorL );
        smashMapper->map ( sliceVectorR );
        break;
    }

    if ( checkAbort() )
        return false;

    std::vector<SliceVectorTransformer<double> *>::iterator svit;
    for ( svit = sliceVectorTransformerVector.begin(); svit != sliceVectorTransformerVector.end(); svit++)
    {
        (*svit)->vectorTransform ( &sliceVectorL, ( channels == 2 ) ? &sliceVectorR : NULL);

        if ( checkAbort() )
            return false;
    }

    std::vector<double> dataOutL, dataOutR;

    switch ( channels )
    {
    case 1:
        retL = sliceVectorToDataCrossFade( sliceVectorL, &dataOut);
        retR = true;
        break;
    case 2:
        retL = sliceVectorToDataCrossFade( sliceVectorL, &dataOutL);
        retR = sliceVectorToDataCrossFade( sliceVectorR, &dataOutR);
        break;
    default:
        retL = sliceVectorToDataCrossFade( sliceVectorL, &dataOutL);
        retR = sliceVectorToDataCrossFade( sliceVectorR, &dataOutR);
        break;
    }

    if ( checkAbort() )
        return false;

    if ( !( retL && retR ) )
        return false;

    std::vector<SliceTransformer<double> *>::iterator stit;
    for ( stit = sliceTransformerVector.begin(); stit != sliceTransformerVector.end(); stit++ )
    {
        (*stit)->transform ( &dataOutL, ( channels == 2 ) ? &dataOutR : NULL );

        if ( checkAbort() )
            return false;
    }

    if ( channels == 2 )
        SampleMath::joinChannels ( &dataOutL, &dataOutR, &dataOut );

    initSliceOffsetVector( requestedSliceNo );

    setState ( WAV16_HAS_DATA_OUT );
    return true;
}

bool Wav16Smash::dataToSliceVectorCrossFade ( std::vector<double> *data, std::vector<std::vector<double> >& sliceVector, size_t requestedSliceNo )
{
    std::vector<double> currentSlice, lastSlice;
    size_t dataSize = data->size();
    size_t sliceSize = dataSize / requestedSliceNo;
    size_t offset, limit;
    offset = limit = 0;
    sliceVector.clear();

    if ( requestedSliceNo > 1 )
    {

        for ( size_t i = 0; i < requestedSliceNo - 1; i ++ )
        {
            limit = offset + sliceSize + crossFadeRange;
            for ( size_t j = offset; j < limit; j++)
                currentSlice.push_back ( data->at ( j ) );

            sliceVector.push_back ( currentSlice );
            currentSlice.clear();
            offset += sliceSize;
        }

    }

    for ( size_t i = offset; i < dataSize; i++)
        lastSlice.push_back ( data->at ( i ) );

    for ( size_t i = 0; i < crossFadeRange; i++)
        lastSlice.push_back ( data->at ( i ) );
    sliceVector.push_back ( lastSlice );

    return true;
}


bool Wav16Smash::sliceVectorToDataCrossFade ( std::vector<std::vector<double> >& sliceVector, std::vector<double> *data )
{
    std::vector<std::vector<double> >::iterator vit;
    size_t dataSize, sliceVectorSize, offset, length, limit, i;
    dataSize = offset = length = limit = 0;
    sliceVectorSize = sliceVector.size();

    for ( vit = sliceVector.begin(); vit != sliceVector.end(); vit++ )
        dataSize += vit->size() - crossFadeRange;

    for ( i = 0; i < dataSize; i++ )
        data->push_back ( 0.0 );

    for ( size_t j = 0; j < sliceVectorSize; j++ )
    {
        length = sliceVector.at ( j ).size();
        limit = length - crossFadeRange;

        i = 0;

        while ( i < crossFadeRange )
        {
            (*data )[ offset + i ] +=
                ( j > 0 ) ?
                lConcaveSoftClipper( i, 0, crossFadeRange ) * sliceVector.at ( j ).at( i ) :
                lConcaveSoftClipper( i, 0, 128 ) * sliceVector.at ( j ).at( i );
            i++;
        }


        while ( ( i >= crossFadeRange ) && ( i < limit  ) )
        {
            (*data )[ offset + i ] += sliceVector.at ( j ).at( i );
            i++;
        }

        while ( i < length )
        {
            (*data )[ offset + i ] +=
                ( j < sliceVectorSize - 1 ) ?
                rConvexSoftClipper ( i, limit, length ) * sliceVector.at ( j ).at( i ):
                rConvexSoftClipper ( i, length - 128, length ) * sliceVector.at ( j ).at( i );
            i++;
        }

        offset += limit;
    }
    return true;

}

void Wav16Smash::initSliceOffsetVector ( size_t requestedSliceNo )
{
    if ( !requestedSliceNo )
        return;

    size_t dataSize, offset, length;
    dataSize = dataOut.size();
    offset = 0;
    length = dataSize / requestedSliceNo;

    sliceOffsetVector.clear();

    for ( size_t i = 0; i < requestedSliceNo - 1; i++ )
    {
        sliceOffsetVector.push_back ( std::make_pair ( offset, length ) );
        offset += length;
        dataSize -= length;
    }

    sliceOffsetVector.push_back ( std::make_pair ( offset, dataSize ) );
}

void Wav16Smash::addSliceVectorTransformer (SliceVectorTransformer<double> *sliceVectorTransformer)
{
    int state = getState();
    if ( state != WAV16_HAS_DATA_IN && state != WAV16_HAS_DATA_OUT )
        return;

    sliceVectorTransformerVector.push_back ( sliceVectorTransformer );
}


void Wav16Smash::addSliceTransformer ( SliceTransformer<double> *sliceTransformer )
{
    int state = getState();
    if ( state != WAV16_HAS_DATA_IN && state != WAV16_HAS_DATA_OUT )
        return;

    sliceTransformerVector.push_back ( sliceTransformer );
}
